"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.default = exports.publish = exports.once = exports.addChannel = void 0;

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _hooks = require("@forrestjs/hooks");

var _pgPubsub = _interopRequireDefault(require("pg-pubsub"));

var hooks = _interopRequireWildcard(require("./hooks"));

const connections = {};

const addChannel = (channelName, fn, connectionName = 'default') => {
  connections[connectionName].addChannel(channelName, fn);
  return () => connections[connectionName].removeChannel(channelName, fn);
};

exports.addChannel = addChannel;

const once = (channelName, fn, connectionName = 'default') => {
  const cb = data => {
    removeChannel(); // eslint-disable-line

    fn(data);
  };

  const removeChannel = addChannel(channelName, cb, connectionName);
};

exports.once = once;

const publish = (channelName, data, connectionName = 'default') => {
  connections[connectionName].publish(channelName, data);
};

exports.publish = publish;

var _default = ({
  registerHook,
  registerAction,
  createHook
}) => {
  registerHook(hooks);
  registerAction({
    hook: _hooks.INIT_FEATURES,
    name: hooks.SERVICE_NAME,
    trace: __filename,
    handler: function () {
      var _handler = (0, _asyncToGenerator2.default)(function* ({
        getConfig
      }) {
        const postgresPubsub = getConfig('postgresPubSub');
        postgresPubsub.forEach(config => {
          const connectionName = config.connectionName || 'default';
          const uri = ['postgres://', `${config.username}:${config.password}`, '@', `${config.host}:${config.port}`, '/', config.database].join('');
          connections[connectionName] = new _pgPubsub.default(uri);
          connections[connectionName].publish('ping', Date.now());
        });
        createHook.sync(hooks.POSTGRES_PUBSUB_START, {
          addChannel,
          once,
          publish
        });
      });

      function handler(_x) {
        return _handler.apply(this, arguments);
      }

      return handler;
    }()
  });
};

exports.default = _default;